# fastlife
# Experimenting with C code and Python bindings using Conway's Game of Life.
#
# Author:   Benjamin Bengfort <benjamin@bengfort.com>
# Created:  Sun Jul 26 08:44:49 2020 -0400
#
# Copyright (C) 2020 Bengfort.com
# For license information, see LICENSE
#
# ID: __init__.py [] benjamin@bengfort.com $

"""
Experimenting with C code and Python bindings using Conway's Game of Life.
"""

##########################################################################
## Imports
##########################################################################

from .version import get_version


__version__ = get_version()
